// Test GameState

#ifndef MAINGAMESTATE_H
#define MAINGAMESTATE_H

#include "../ToolKit/Singleton.h"
#include "../StateModule/GameState.h"

#include "../Timer/FpsCounter.h"

// Forward declare classes
namespace GfxModule
{
    class BGObject;
    class Sprite;
    class FontWrapper;
}

namespace SoundModule
{
	class ChunkWrapper;
}

class MainGameState : public GameStateModule::GameState, public Singleton<MainGameState>
{
private:
    GfxModule::Sprite * _KenSprite;
    GfxModule::BGObject * _MarioBG, * _CircleBG, * _HoleBG;
    GfxModule::FontWrapper * _CourierNewFont;
    SoundModule::ChunkWrapper * _DrumRollSound, * _GlassBreakSound;
    TimerModule::FpsCounter _FpsCounter;

public:
    MainGameState(void) :
        _KenSprite(0),
        _MarioBG(0),
        _CircleBG(0),
        _HoleBG(0),
        _CourierNewFont(0),
        _DrumRollSound(0),
        _GlassBreakSound(0)
    {
        // Intentionally empty
    }

    ~MainGameState(void)
    {
        // Intentionally empty
    }

    //! Pure virtual. Initialise the state when the state is pushed on top the stack
    void PushInitialise(
        GfxModule::GfxModuleWrapper* AGfxModule,
        SoundModule::SoundModuleWrapper* ASoundModule
    );

    //! Pure virtual. Cleanup up the state when it is pushed then further into the stack
    void PushCleanup(
        GfxModule::GfxModuleWrapper* AGfxModule,
        SoundModule::SoundModuleWrapper* ASoundModule
    );

    //! Pure virtual. When the state is popped up to the top of the stack
    void PopInitialise(
        GfxModule::GfxModuleWrapper* AGfxModule,
        SoundModule::SoundModuleWrapper* ASoundModule
    );

    //! Pure virtual. When the state is popped off the stack
    void PopCleanup(
        GfxModule::GfxModuleWrapper* AGfxModule,
        SoundModule::SoundModuleWrapper* ASoundModule
    );
    //! Pure virtual. Gets called once per frame when it is on top of the stack
    void Update(
        GfxModule::GfxModuleWrapper* AGfxModule,
        SoundModule::SoundModuleWrapper* ASoundModule,
        InputModule::Input* AInputModule,
        GameStateModule::GameStateManager* AGameStateManager
    );
};

#endif // MAINGAMESTATE_H
