#include <fstream>
#include <sstream>
#include <string>

#include "../InputModule/Input.h"

#include "../GfxModule/GfxModuleWrapper.h"
#include "../GfxModule/FontManager.h"

#include "../SoundModule/SoundModuleWrapper.h"
#include "../SoundModule/MusicManager.h"

#include "../StateModule/GameStateManager.h"
#include "../ToolKit/SdlResize.h"
#include "./MusicState.h"

// A lot fo the wav display code is taken from the SDL Mixer demo

using namespace GfxModule;
using namespace SoundModule;
using namespace InputModule;
using namespace GameStateModule;

using namespace std;

void MusicState::PushInitialise(
    GfxModuleWrapper* AGfxModule,
    SoundModuleWrapper* ASoundModule
)
{
    // Load the font
    _CourierNewFont = AGfxModule->GetFontManager()->CreateFont("./Courier_New_Bold.ttf", 12);

    // Load the music from a file list
    ifstream fin("SongList.cfg");
    string SongFileName = "";
    getline(fin, SongFileName, ';');
    ASoundModule->GetMusicManager()->LoadMusic(SongFileName.c_str() );
}

//! Pure virtual. Cleanup up the state when it is pushed then further into the stack
void MusicState::PushCleanup(
    GfxModuleWrapper* AGfxModule,
    SoundModuleWrapper* ASoundModule
)
{
    // Free everything
    AGfxModule->GetFontManager()->FreeFont(_CourierNewFont);
    ASoundModule->GetMusicManager()->FreeMusic();
}

//! Pure virtual. When the state is popped up to the top of the stack
void MusicState::PopInitialise(
    GfxModuleWrapper* AGfxModule,
    SoundModuleWrapper* ASoundModule
)
{
    this->PushInitialise(AGfxModule, ASoundModule);
}

//! Pure virtual. When the state is popped off the stack
void MusicState::PopCleanup(
    GfxModuleWrapper* AGfxModule,
    SoundModuleWrapper* ASoundModule
)
{
    this->PushCleanup(AGfxModule, ASoundModule);
}

//! Pure virtual. Gets called once per frame when it is on top of the stack
void MusicState::Update(
    GfxModuleWrapper* AGfxModule,
    SoundModuleWrapper* ASoundModule,
    Input* AInputModule,
    GameStateManager* AGameStateManager
)
{
    // Update the input
    AInputModule->UpdateInputState();
    unsigned int CurrentInputState = AInputModule->GetCurrentInputState();
    unsigned int PreviousInputState = AInputModule->GetPreviousInputState();

    if ( CurrentInputState & BUTTON_Y )
    {
        if ( !(PreviousInputState & BUTTON_Y ) )
           ASoundModule->GetMusicManager()->Stop();
    }

    if ( CurrentInputState & BUTTON_A )
    {
        if ( !(PreviousInputState & BUTTON_A ) )
           ASoundModule->GetMusicManager()->Play(1);
    }

    if ( CurrentInputState & BUTTON_B )
    {
        if ( !(PreviousInputState & BUTTON_B ) )
           ASoundModule->GetMusicManager()->Pause();
    }

    if ( CurrentInputState & BUTTON_X )
    {
        if ( !(PreviousInputState & BUTTON_X ) )
           ASoundModule->GetMusicManager()->Resume();
    }

    if (CurrentInputState & BUTTON_VOLDOWN )
    {
        ASoundModule->GetMusicManager()->VolumeDown();
    }

    if (CurrentInputState & BUTTON_VOLUP )
    {
        ASoundModule->GetMusicManager()->VolumeUp();
    }

    // Update the FPS counter
    _FpsCounter.Update();

    // Print the FPS count in the top right in white
    unsigned int CurrentFps = _FpsCounter.GetFpsCount();
    ostringstream FpsOStringStream;
    FpsOStringStream << "FPS: " << CurrentFps;
    AGfxModule->GetFontManager()->WriteText(_CourierNewFont, 1, 1, FpsOStringStream.str(), 255, 255, 255);

    if ( CurrentInputState & BUTTON_JOYCLICK )
    {
        // Quit the game
        if ( !(PreviousInputState & BUTTON_JOYCLICK) )
           AGameStateManager->Pop();
    }
}
