#include "../Kernel/Kernel.h"
#include "../Logger/Logger.h"

#include "../StateModule/GameStateManager.h"
#include "./MainGame.h"
#include "./MusicState.h"

using namespace KernelModule;
using namespace GfxModule;
using namespace TimerModule;
using namespace GameStateModule;

int main ( int argc, char** argv )
{
	// Create a new Kernel
	Kernel *TheKernel = new Kernel(
        320,                    // Screen Width
        240,                    // Screen Height
        16,                     // Colour depth
        DEFAULT_VIDEO_MODE,     // Default video mode/SDL flags
        128,                    // Number of Surfaces
        4,                      // Number of Layers
        128,                    // Number of Sprites
        30,                     // FPS Cap
        16,						// Number of Sound effects
        4,						// Number of Sound channels
        4						// Number of Fonts
    );

    // Check the Kernel Status
    if( !TheKernel->GetStatus() )
    {
		// If bad, Bail
		return -1;
	}

    // Create a new State Manager [Singleton]
    GameStateManager * TheGameStateManager = new GameStateManager(TheKernel);

    // Create a new state
    new MainGameState();
    new MusicState();

    // Add a new state
    TheGameStateManager->Push( MainGameState::GetSingletonPtr() );

	while( !TheGameStateManager->Empty() )
	{
        TheGameStateManager->Update();
	}

	// Free the memory
	delete MusicState::GetSingletonPtr();
	delete MainGameState::GetSingletonPtr();
	delete TheGameStateManager;
	delete TheKernel;

	// Return to the menu
	#ifdef GP2X
    chdir("/usr/gp2x");
	execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
	#endif // GP2X

	return 0;
}
